# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .who_can_post_types import WhoCanPostTypes
from .who_can_comment_types import WhoCanCommentTypes
from .email_notification_preferences import EmailNotificationPreferences

__all__ = ["Forum", "Experience"]


class Experience(BaseModel):
    id: str
    """The unique ID representing this experience"""

    name: str
    """The written name of the description."""


class Forum(BaseModel):
    id: str
    """The unique identifier for the entity"""

    email_notification_preference: EmailNotificationPreferences
    """The email notification preference for this forum"""

    experience: Experience
    """The experience for this forum"""

    who_can_comment: WhoCanCommentTypes
    """Who can comment on this forum"""

    who_can_post: WhoCanPostTypes
    """Who can post on this forum"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import (
    MembershipListResponse,
)
from whop_sdk._utils import parse_datetime
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import Membership

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMemberships:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        membership = client.memberships.retrieve(
            "mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.memberships.with_raw_response.retrieve(
            "mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.memberships.with_streaming_response.retrieve(
            "mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.memberships.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Whop) -> None:
        membership = client.memberships.update(
            id="mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Whop) -> None:
        membership = client.memberships.update(
            id="mem_xxxxxxxxxxxxxx",
            metadata={"foo": "bar"},
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Whop) -> None:
        response = client.memberships.with_raw_response.update(
            id="mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Whop) -> None:
        with client.memberships.with_streaming_response.update(
            id="mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.memberships.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        membership = client.memberships.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[MembershipListResponse], membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        membership = client.memberships.list(
            company_id="biz_xxxxxxxxxxxxxx",
            access_pass_ids=["string"],
            after="after",
            before="before",
            cancel_options=["too_expensive"],
            created_after=parse_datetime("2023-12-01T05:00:00.401Z"),
            created_before=parse_datetime("2023-12-01T05:00:00.401Z"),
            direction="asc",
            first=42,
            last=42,
            order="id",
            plan_ids=["string"],
            promo_code_ids=["string"],
            statuses=["trialing"],
        )
        assert_matches_type(SyncCursorPage[MembershipListResponse], membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.memberships.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = response.parse()
        assert_matches_type(SyncCursorPage[MembershipListResponse], membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.memberships.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = response.parse()
            assert_matches_type(SyncCursorPage[MembershipListResponse], membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_cancel(self, client: Whop) -> None:
        membership = client.memberships.cancel(
            id="mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_cancel_with_all_params(self, client: Whop) -> None:
        membership = client.memberships.cancel(
            id="mem_xxxxxxxxxxxxxx",
            cancellation_mode="at_period_end",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_cancel(self, client: Whop) -> None:
        response = client.memberships.with_raw_response.cancel(
            id="mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_cancel(self, client: Whop) -> None:
        with client.memberships.with_streaming_response.cancel(
            id="mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_cancel(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.memberships.with_raw_response.cancel(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_pause(self, client: Whop) -> None:
        membership = client.memberships.pause(
            id="mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_pause_with_all_params(self, client: Whop) -> None:
        membership = client.memberships.pause(
            id="mem_xxxxxxxxxxxxxx",
            void_payments=True,
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_pause(self, client: Whop) -> None:
        response = client.memberships.with_raw_response.pause(
            id="mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_pause(self, client: Whop) -> None:
        with client.memberships.with_streaming_response.pause(
            id="mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_pause(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.memberships.with_raw_response.pause(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_resume(self, client: Whop) -> None:
        membership = client.memberships.resume(
            "mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_resume(self, client: Whop) -> None:
        response = client.memberships.with_raw_response.resume(
            "mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_resume(self, client: Whop) -> None:
        with client.memberships.with_streaming_response.resume(
            "mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_resume(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.memberships.with_raw_response.resume(
                "",
            )


class TestAsyncMemberships:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.retrieve(
            "mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.memberships.with_raw_response.retrieve(
            "mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = await response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.memberships.with_streaming_response.retrieve(
            "mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = await response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.memberships.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.update(
            id="mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.update(
            id="mem_xxxxxxxxxxxxxx",
            metadata={"foo": "bar"},
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncWhop) -> None:
        response = await async_client.memberships.with_raw_response.update(
            id="mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = await response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncWhop) -> None:
        async with async_client.memberships.with_streaming_response.update(
            id="mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = await response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.memberships.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[MembershipListResponse], membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.list(
            company_id="biz_xxxxxxxxxxxxxx",
            access_pass_ids=["string"],
            after="after",
            before="before",
            cancel_options=["too_expensive"],
            created_after=parse_datetime("2023-12-01T05:00:00.401Z"),
            created_before=parse_datetime("2023-12-01T05:00:00.401Z"),
            direction="asc",
            first=42,
            last=42,
            order="id",
            plan_ids=["string"],
            promo_code_ids=["string"],
            statuses=["trialing"],
        )
        assert_matches_type(AsyncCursorPage[MembershipListResponse], membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.memberships.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = await response.parse()
        assert_matches_type(AsyncCursorPage[MembershipListResponse], membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.memberships.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = await response.parse()
            assert_matches_type(AsyncCursorPage[MembershipListResponse], membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_cancel(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.cancel(
            id="mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_cancel_with_all_params(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.cancel(
            id="mem_xxxxxxxxxxxxxx",
            cancellation_mode="at_period_end",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncWhop) -> None:
        response = await async_client.memberships.with_raw_response.cancel(
            id="mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = await response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncWhop) -> None:
        async with async_client.memberships.with_streaming_response.cancel(
            id="mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = await response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.memberships.with_raw_response.cancel(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_pause(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.pause(
            id="mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_pause_with_all_params(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.pause(
            id="mem_xxxxxxxxxxxxxx",
            void_payments=True,
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_pause(self, async_client: AsyncWhop) -> None:
        response = await async_client.memberships.with_raw_response.pause(
            id="mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = await response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_pause(self, async_client: AsyncWhop) -> None:
        async with async_client.memberships.with_streaming_response.pause(
            id="mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = await response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_pause(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.memberships.with_raw_response.pause(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_resume(self, async_client: AsyncWhop) -> None:
        membership = await async_client.memberships.resume(
            "mem_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_resume(self, async_client: AsyncWhop) -> None:
        response = await async_client.memberships.with_raw_response.resume(
            "mem_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        membership = await response.parse()
        assert_matches_type(Membership, membership, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_resume(self, async_client: AsyncWhop) -> None:
        async with async_client.memberships.with_streaming_response.resume(
            "mem_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            membership = await response.parse()
            assert_matches_type(Membership, membership, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_resume(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.memberships.with_raw_response.resume(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["AccessPassType"]

AccessPassType: TypeAlias = Literal["regular", "app", "experience_upsell", "api_only"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .custom_cta import CustomCta
from .visibility import Visibility
from .business_types import BusinessTypes
from .industry_types import IndustryTypes
from .global_affiliate_status import GlobalAffiliateStatus

__all__ = ["Product", "Company", "OwnerUser", "ProductTaxCode"]


class Company(BaseModel):
    id: str
    """The ID (tag) of the company."""

    route: str
    """The slug/route of the company on the Whop site."""

    title: str
    """The title of the company."""


class OwnerUser(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class ProductTaxCode(BaseModel):
    id: str
    """The internal ID of the product tax code."""

    name: str
    """The name of the product tax code."""

    product_type: Literal["physical", "digital", "services"]
    """The type of product this tax code applies to."""


class Product(BaseModel):
    id: str
    """The internal ID of the public product."""

    business_type: Optional[BusinessTypes] = None
    """The different business types a company can be."""

    company: Company
    """A short type of the company that this product belongs to."""

    created_at: datetime
    """When the product was created."""

    custom_cta: CustomCta
    """The custom call to action for the product."""

    custom_cta_url: Optional[str] = None
    """The custom call to action URL for the product, if any."""

    custom_statement_descriptor: Optional[str] = None
    """The custom statement descriptor for the product."""

    description: Optional[str] = None
    """A short description of what the company offers or does."""

    global_affiliate_percentage: Optional[float] = None
    """
    The percentage of a transaction a user is eligible to earn from the whop
    marketplace global affiliate program.
    """

    global_affiliate_status: GlobalAffiliateStatus
    """The status of the global affiliate program for this product."""

    headline: Optional[str] = None
    """The headline of the product."""

    industry_type: Optional[IndustryTypes] = None
    """The different industry types a company can be in."""

    member_affiliate_percentage: Optional[float] = None
    """
    The percentage of a transaction a user is eligible to earn from the whop
    marketplace member affiliate program.
    """

    member_affiliate_status: GlobalAffiliateStatus
    """The status of the member affiliate program for this product."""

    member_count: int
    """The number of active users for this product."""

    owner_user: OwnerUser
    """The user that owns the product (company owner)."""

    product_tax_code: Optional[ProductTaxCode] = None
    """The product tax code for the product, if any."""

    published_reviews_count: int
    """The number of reviews that have been published for the product."""

    route: str
    """The route of the product."""

    title: str
    """The title of the product. Use for Whop 4.0."""

    updated_at: datetime
    """When the product was updated."""

    verified: bool
    """Whether this product is Whop verified."""

    visibility: Visibility
    """This product will/will not be displayed publicly."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .shared.access_level import AccessLevel

__all__ = ["UserCheckAccessResponse"]


class UserCheckAccessResponse(BaseModel):
    access_level: AccessLevel
    """The permission level of the user"""

    has_access: bool
    """Whether the user has access to the resource"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import (
    App as App,
    Plan as Plan,
    Entry as Entry,
    Forum as Forum,
    Company as Company,
    Invoice as Invoice,
    Message as Message,
    Payment as Payment,
    Product as Product,
    TaxType as TaxType,
    AppBuild as AppBuild,
    Currency as Currency,
    PageInfo as PageInfo,
    PlanType as PlanType,
    Reaction as Reaction,
    Shipment as Shipment,
    Transfer as Transfer,
    CustomCta as CustomCta,
    Direction as Direction,
    ForumPost as ForumPost,
    PromoType as PromoType,
    Experience as Experience,
    Membership as Membership,
    Visibility as Visibility,
    WhoCanPost as WhoCanPost,
    AccessLevel as AccessLevel,
    AppStatuses as AppStatuses,
    AppViewType as AppViewType,
    ChatChannel as ChatChannel,
    EntryStatus as EntryStatus,
    WhoCanReact as WhoCanReact,
    DmsPostTypes as DmsPostTypes,
    BusinessTypes as BusinessTypes,
    IndustryTypes as IndustryTypes,
    InvoiceStatus as InvoiceStatus,
    ReceiptStatus as ReceiptStatus,
    ReleaseMethod as ReleaseMethod,
    AccessPassType as AccessPassType,
    MemberStatuses as MemberStatuses,
    ShipmentStatus as ShipmentStatus,
    SupportChannel as SupportChannel,
    InvoiceListItem as InvoiceListItem,
    ProductListItem as ProductListItem,
    ShipmentCarrier as ShipmentCarrier,
    WhoCanPostTypes as WhoCanPostTypes,
    AppBuildStatuses as AppBuildStatuses,
    CollectionMethod as CollectionMethod,
    MembershipStatus as MembershipStatus,
    VisibilityFilter as VisibilityFilter,
    AppBuildPlatforms as AppBuildPlatforms,
    ShipmentSubstatus as ShipmentSubstatus,
    WhoCanCommentTypes as WhoCanCommentTypes,
    AuthorizedUserRoles as AuthorizedUserRoles,
    CheckoutConfiguration as CheckoutConfiguration,
    FriendlyReceiptStatus as FriendlyReceiptStatus,
    GlobalAffiliateStatus as GlobalAffiliateStatus,
    CourseLessonInteraction as CourseLessonInteraction,
    MemberMostRecentActions as MemberMostRecentActions,
    EmailNotificationPreferences as EmailNotificationPreferences,
    CourseLessonInteractionListItem as CourseLessonInteractionListItem,
)
from .app_list_params import AppListParams as AppListParams
from .plan_list_params import PlanListParams as PlanListParams
from .app_create_params import AppCreateParams as AppCreateParams
from .app_list_response import AppListResponse as AppListResponse
from .app_update_params import AppUpdateParams as AppUpdateParams
from .entry_list_params import EntryListParams as EntryListParams
from .forum_list_params import ForumListParams as ForumListParams
from .member_list_params import MemberListParams as MemberListParams
from .plan_create_params import PlanCreateParams as PlanCreateParams
from .plan_list_response import PlanListResponse as PlanListResponse
from .plan_update_params import PlanUpdateParams as PlanUpdateParams
from .entry_list_response import EntryListResponse as EntryListResponse
from .forum_list_response import ForumListResponse as ForumListResponse
from .forum_update_params import ForumUpdateParams as ForumUpdateParams
from .invoice_list_params import InvoiceListParams as InvoiceListParams
from .message_list_params import MessageListParams as MessageListParams
from .payment_list_params import PaymentListParams as PaymentListParams
from .product_list_params import ProductListParams as ProductListParams
from .member_list_response import MemberListResponse as MemberListResponse
from .plan_delete_response import PlanDeleteResponse as PlanDeleteResponse
from .reaction_list_params import ReactionListParams as ReactionListParams
from .shipment_list_params import ShipmentListParams as ShipmentListParams
from .transfer_list_params import TransferListParams as TransferListParams
from .unwrap_webhook_event import UnwrapWebhookEvent as UnwrapWebhookEvent
from .app_build_list_params import AppBuildListParams as AppBuildListParams
from .invoice_create_params import InvoiceCreateParams as InvoiceCreateParams
from .invoice_void_response import InvoiceVoidResponse as InvoiceVoidResponse
from .message_create_params import MessageCreateParams as MessageCreateParams
from .message_list_response import MessageListResponse as MessageListResponse
from .payment_list_response import PaymentListResponse as PaymentListResponse
from .payment_refund_params import PaymentRefundParams as PaymentRefundParams
from .product_create_params import ProductCreateParams as ProductCreateParams
from .product_update_params import ProductUpdateParams as ProductUpdateParams
from .entry_approve_response import EntryApproveResponse as EntryApproveResponse
from .experience_list_params import ExperienceListParams as ExperienceListParams
from .forum_post_list_params import ForumPostListParams as ForumPostListParams
from .membership_list_params import MembershipListParams as MembershipListParams
from .reaction_create_params import ReactionCreateParams as ReactionCreateParams
from .reaction_list_response import ReactionListResponse as ReactionListResponse
from .shipment_create_params import ShipmentCreateParams as ShipmentCreateParams
from .shipment_list_response import ShipmentListResponse as ShipmentListResponse
from .transfer_create_params import TransferCreateParams as TransferCreateParams
from .transfer_list_response import TransferListResponse as TransferListResponse
from .user_retrieve_response import UserRetrieveResponse as UserRetrieveResponse
from .app_build_create_params import AppBuildCreateParams as AppBuildCreateParams
from .app_build_list_response import AppBuildListResponse as AppBuildListResponse
from .invoice_create_response import InvoiceCreateResponse as InvoiceCreateResponse
from .membership_pause_params import MembershipPauseParams as MembershipPauseParams
from .product_delete_response import ProductDeleteResponse as ProductDeleteResponse
from .chat_channel_list_params import ChatChannelListParams as ChatChannelListParams
from .experience_attach_params import ExperienceAttachParams as ExperienceAttachParams
from .experience_create_params import ExperienceCreateParams as ExperienceCreateParams
from .experience_detach_params import ExperienceDetachParams as ExperienceDetachParams
from .experience_list_response import ExperienceListResponse as ExperienceListResponse
from .experience_update_params import ExperienceUpdateParams as ExperienceUpdateParams
from .forum_post_create_params import ForumPostCreateParams as ForumPostCreateParams
from .forum_post_list_response import ForumPostListResponse as ForumPostListResponse
from .member_retrieve_response import MemberRetrieveResponse as MemberRetrieveResponse
from .membership_cancel_params import MembershipCancelParams as MembershipCancelParams
from .membership_list_response import MembershipListResponse as MembershipListResponse
from .membership_update_params import MembershipUpdateParams as MembershipUpdateParams
from .chat_channel_list_response import ChatChannelListResponse as ChatChannelListResponse
from .chat_channel_update_params import ChatChannelUpdateParams as ChatChannelUpdateParams
from .entry_denied_webhook_event import EntryDeniedWebhookEvent as EntryDeniedWebhookEvent
from .experience_delete_response import ExperienceDeleteResponse as ExperienceDeleteResponse
from .invoice_paid_webhook_event import InvoicePaidWebhookEvent as InvoicePaidWebhookEvent
from .user_check_access_response import UserCheckAccessResponse as UserCheckAccessResponse
from .authorized_user_list_params import AuthorizedUserListParams as AuthorizedUserListParams
from .entry_created_webhook_event import EntryCreatedWebhookEvent as EntryCreatedWebhookEvent
from .entry_deleted_webhook_event import EntryDeletedWebhookEvent as EntryDeletedWebhookEvent
from .support_channel_list_params import SupportChannelListParams as SupportChannelListParams
from .entry_approved_webhook_event import EntryApprovedWebhookEvent as EntryApprovedWebhookEvent
from .invoice_voided_webhook_event import InvoiceVoidedWebhookEvent as InvoiceVoidedWebhookEvent
from .payment_failed_webhook_event import PaymentFailedWebhookEvent as PaymentFailedWebhookEvent
from .authorized_user_list_response import AuthorizedUserListResponse as AuthorizedUserListResponse
from .invoice_created_webhook_event import InvoiceCreatedWebhookEvent as InvoiceCreatedWebhookEvent
from .payment_pending_webhook_event import PaymentPendingWebhookEvent as PaymentPendingWebhookEvent
from .support_channel_create_params import SupportChannelCreateParams as SupportChannelCreateParams
from .support_channel_list_response import SupportChannelListResponse as SupportChannelListResponse
from .invoice_past_due_webhook_event import InvoicePastDueWebhookEvent as InvoicePastDueWebhookEvent
from .payment_succeeded_webhook_event import PaymentSucceededWebhookEvent as PaymentSucceededWebhookEvent
from .ledger_account_retrieve_response import LedgerAccountRetrieveResponse as LedgerAccountRetrieveResponse
from .authorized_user_retrieve_response import AuthorizedUserRetrieveResponse as AuthorizedUserRetrieveResponse
from .checkout_configuration_list_params import CheckoutConfigurationListParams as CheckoutConfigurationListParams
from .membership_activated_webhook_event import MembershipActivatedWebhookEvent as MembershipActivatedWebhookEvent
from .checkout_configuration_create_params import CheckoutConfigurationCreateParams as CheckoutConfigurationCreateParams
from .checkout_configuration_list_response import CheckoutConfigurationListResponse as CheckoutConfigurationListResponse
from .membership_deactivated_webhook_event import MembershipDeactivatedWebhookEvent as MembershipDeactivatedWebhookEvent
from .course_lesson_interaction_list_params import (
    CourseLessonInteractionListParams as CourseLessonInteractionListParams,
)
from .course_lesson_interaction_completed_webhook_event import (
    CourseLessonInteractionCompletedWebhookEvent as CourseLessonInteractionCompletedWebhookEvent,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import AppBuildListResponse
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import AppBuild

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAppBuilds:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Whop) -> None:
        app_build = client.app_builds.create(
            attachment={},
            checksum="checksum",
            platform="ios",
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Whop) -> None:
        app_build = client.app_builds.create(
            attachment={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            checksum="checksum",
            platform="ios",
            app_id="app_xxxxxxxxxxxxxx",
            supported_app_view_types=["hub"],
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Whop) -> None:
        response = client.app_builds.with_raw_response.create(
            attachment={},
            checksum="checksum",
            platform="ios",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = response.parse()
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Whop) -> None:
        with client.app_builds.with_streaming_response.create(
            attachment={},
            checksum="checksum",
            platform="ios",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = response.parse()
            assert_matches_type(AppBuild, app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        app_build = client.app_builds.retrieve(
            "apbu_xxxxxxxxxxxxx",
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.app_builds.with_raw_response.retrieve(
            "apbu_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = response.parse()
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.app_builds.with_streaming_response.retrieve(
            "apbu_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = response.parse()
            assert_matches_type(AppBuild, app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.app_builds.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        app_build = client.app_builds.list(
            app_id="app_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[AppBuildListResponse], app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        app_build = client.app_builds.list(
            app_id="app_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            first=42,
            last=42,
            platform="ios",
            status="draft",
        )
        assert_matches_type(SyncCursorPage[AppBuildListResponse], app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.app_builds.with_raw_response.list(
            app_id="app_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = response.parse()
        assert_matches_type(SyncCursorPage[AppBuildListResponse], app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.app_builds.with_streaming_response.list(
            app_id="app_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = response.parse()
            assert_matches_type(SyncCursorPage[AppBuildListResponse], app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_promote(self, client: Whop) -> None:
        app_build = client.app_builds.promote(
            "apbu_xxxxxxxxxxxxx",
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_promote(self, client: Whop) -> None:
        response = client.app_builds.with_raw_response.promote(
            "apbu_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = response.parse()
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_promote(self, client: Whop) -> None:
        with client.app_builds.with_streaming_response.promote(
            "apbu_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = response.parse()
            assert_matches_type(AppBuild, app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_promote(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.app_builds.with_raw_response.promote(
                "",
            )


class TestAsyncAppBuilds:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncWhop) -> None:
        app_build = await async_client.app_builds.create(
            attachment={},
            checksum="checksum",
            platform="ios",
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncWhop) -> None:
        app_build = await async_client.app_builds.create(
            attachment={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            checksum="checksum",
            platform="ios",
            app_id="app_xxxxxxxxxxxxxx",
            supported_app_view_types=["hub"],
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncWhop) -> None:
        response = await async_client.app_builds.with_raw_response.create(
            attachment={},
            checksum="checksum",
            platform="ios",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = await response.parse()
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncWhop) -> None:
        async with async_client.app_builds.with_streaming_response.create(
            attachment={},
            checksum="checksum",
            platform="ios",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = await response.parse()
            assert_matches_type(AppBuild, app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        app_build = await async_client.app_builds.retrieve(
            "apbu_xxxxxxxxxxxxx",
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.app_builds.with_raw_response.retrieve(
            "apbu_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = await response.parse()
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.app_builds.with_streaming_response.retrieve(
            "apbu_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = await response.parse()
            assert_matches_type(AppBuild, app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.app_builds.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        app_build = await async_client.app_builds.list(
            app_id="app_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[AppBuildListResponse], app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        app_build = await async_client.app_builds.list(
            app_id="app_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            first=42,
            last=42,
            platform="ios",
            status="draft",
        )
        assert_matches_type(AsyncCursorPage[AppBuildListResponse], app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.app_builds.with_raw_response.list(
            app_id="app_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = await response.parse()
        assert_matches_type(AsyncCursorPage[AppBuildListResponse], app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.app_builds.with_streaming_response.list(
            app_id="app_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = await response.parse()
            assert_matches_type(AsyncCursorPage[AppBuildListResponse], app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_promote(self, async_client: AsyncWhop) -> None:
        app_build = await async_client.app_builds.promote(
            "apbu_xxxxxxxxxxxxx",
        )
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_promote(self, async_client: AsyncWhop) -> None:
        response = await async_client.app_builds.with_raw_response.promote(
            "apbu_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        app_build = await response.parse()
        assert_matches_type(AppBuild, app_build, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_promote(self, async_client: AsyncWhop) -> None:
        async with async_client.app_builds.with_streaming_response.promote(
            "apbu_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            app_build = await response.parse()
            assert_matches_type(AppBuild, app_build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_promote(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.app_builds.with_raw_response.promote(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .currency import Currency as Currency
from .tax_type import TaxType as TaxType
from .direction import Direction as Direction
from .plan_type import PlanType as PlanType
from .custom_cta import CustomCta as CustomCta
from .visibility import Visibility as Visibility
from .access_level import AccessLevel as AccessLevel
from .app_statuses import AppStatuses as AppStatuses
from .entry_status import EntryStatus as EntryStatus
from .who_can_post import WhoCanPost as WhoCanPost
from .app_view_type import AppViewType as AppViewType
from .who_can_react import WhoCanReact as WhoCanReact
from .business_types import BusinessTypes as BusinessTypes
from .industry_types import IndustryTypes as IndustryTypes
from .invoice_status import InvoiceStatus as InvoiceStatus
from .receipt_status import ReceiptStatus as ReceiptStatus
from .release_method import ReleaseMethod as ReleaseMethod
from .member_statuses import MemberStatuses as MemberStatuses
from .access_pass_type import AccessPassType as AccessPassType
from .collection_method import CollectionMethod as CollectionMethod
from .membership_status import MembershipStatus as MembershipStatus
from .visibility_filter import VisibilityFilter as VisibilityFilter
from .app_build_statuses import AppBuildStatuses as AppBuildStatuses
from .who_can_post_types import WhoCanPostTypes as WhoCanPostTypes
from .app_build_platforms import AppBuildPlatforms as AppBuildPlatforms
from .authorized_user_roles import AuthorizedUserRoles as AuthorizedUserRoles
from .who_can_comment_types import WhoCanCommentTypes as WhoCanCommentTypes
from .friendly_receipt_status import FriendlyReceiptStatus as FriendlyReceiptStatus
from .global_affiliate_status import GlobalAffiliateStatus as GlobalAffiliateStatus
from .member_most_recent_actions import MemberMostRecentActions as MemberMostRecentActions
from .email_notification_preferences import EmailNotificationPreferences as EmailNotificationPreferences



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["ForumPostListParams"]


class ForumPostListParams(TypedDict, total=False):
    experience_id: Required[str]
    """The ID of the experience to list forum posts for"""

    after: Optional[str]
    """Returns the elements in the list that come after the specified cursor."""

    before: Optional[str]
    """Returns the elements in the list that come before the specified cursor."""

    first: Optional[int]
    """Returns the first _n_ elements from the list."""

    last: Optional[int]
    """Returns the last _n_ elements from the list."""

    parent_id: Optional[str]
    """The ID of the parent post to list forum post comments for"""

    pinned: Optional[bool]
    """Set to true to only return pinned posts"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["AppViewType"]

AppViewType: TypeAlias = Literal["hub", "discover", "dash", "dashboard", "analytics"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    experience_list_params,
    experience_attach_params,
    experience_create_params,
    experience_detach_params,
    experience_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.experience import Experience
from ..types.experience_list_response import ExperienceListResponse
from ..types.experience_delete_response import ExperienceDeleteResponse

__all__ = ["ExperiencesResource", "AsyncExperiencesResource"]


class ExperiencesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExperiencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return ExperiencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExperiencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return ExperiencesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        app_id: str,
        company_id: str,
        name: Optional[str] | Omit = omit,
        section_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Required permissions:

        - `experience:create`

        Args:
          app_id: The ID of the app to create the experience for

          company_id: The ID of the company to create the experience for

          name: The name of the experience

          section_id: The ID of the section to create the experience in

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/experiences",
            body=maybe_transform(
                {
                    "app_id": app_id,
                    "company_id": company_id,
                    "name": name,
                    "section_id": section_id,
                },
                experience_create_params.ExperienceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Retrieves an experience by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/experiences/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    def update(
        self,
        id: str,
        *,
        access_level: Optional[Literal["public", "private"]] | Omit = omit,
        logo: Optional[experience_update_params.Logo] | Omit = omit,
        name: Optional[str] | Omit = omit,
        order: Optional[str] | Omit = omit,
        section_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Required permissions:

        - `experience:update`

        Args:
          access_level: The different access levels for experiences (PUBLIC IS NEVER USED ANYMORE).

          logo: The logo for the experience

          name: The name of the experience.

          order: The order of the experience in the section.

          section_id: The ID of the section to update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/experiences/{id}",
            body=maybe_transform(
                {
                    "access_level": access_level,
                    "logo": logo,
                    "name": name,
                    "order": order,
                    "section_id": section_id,
                },
                experience_update_params.ExperienceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        app_id: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        product_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[ExperienceListResponse]:
        """
        Lists experiences for a company

        Required permissions:

        - `experience:hidden_experience:read`

        Args:
          company_id: The ID of the company to filter experiences by

          after: Returns the elements in the list that come after the specified cursor.

          app_id: The ID of the app to filter experiences by

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          product_id: The ID of the product to filter experiences by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/experiences",
            page=SyncCursorPage[ExperienceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "app_id": app_id,
                        "before": before,
                        "first": first,
                        "last": last,
                        "product_id": product_id,
                    },
                    experience_list_params.ExperienceListParams,
                ),
            ),
            model=ExperienceListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExperienceDeleteResponse:
        """
        Required permissions:

        - `experience:delete`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/experiences/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceDeleteResponse,
        )

    def attach(
        self,
        id: str,
        *,
        product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Adds an experience to an product, making it accessible to the product's
        customers.

        Required permissions:

        - `experience:attach`

        Args:
          product_id: The ID of the Access Pass to add the Experience to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/experiences/{id}/attach",
            body=maybe_transform({"product_id": product_id}, experience_attach_params.ExperienceAttachParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    def detach(
        self,
        id: str,
        *,
        product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Removes an experience from an product, making it inaccessible to the product's
        customers.

        Required permissions:

        - `experience:detach`

        Args:
          product_id: The ID of the Access Pass to add the Experience to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/experiences/{id}/detach",
            body=maybe_transform({"product_id": product_id}, experience_detach_params.ExperienceDetachParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )


class AsyncExperiencesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExperiencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExperiencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExperiencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncExperiencesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        app_id: str,
        company_id: str,
        name: Optional[str] | Omit = omit,
        section_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Required permissions:

        - `experience:create`

        Args:
          app_id: The ID of the app to create the experience for

          company_id: The ID of the company to create the experience for

          name: The name of the experience

          section_id: The ID of the section to create the experience in

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/experiences",
            body=await async_maybe_transform(
                {
                    "app_id": app_id,
                    "company_id": company_id,
                    "name": name,
                    "section_id": section_id,
                },
                experience_create_params.ExperienceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Retrieves an experience by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/experiences/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    async def update(
        self,
        id: str,
        *,
        access_level: Optional[Literal["public", "private"]] | Omit = omit,
        logo: Optional[experience_update_params.Logo] | Omit = omit,
        name: Optional[str] | Omit = omit,
        order: Optional[str] | Omit = omit,
        section_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Required permissions:

        - `experience:update`

        Args:
          access_level: The different access levels for experiences (PUBLIC IS NEVER USED ANYMORE).

          logo: The logo for the experience

          name: The name of the experience.

          order: The order of the experience in the section.

          section_id: The ID of the section to update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/experiences/{id}",
            body=await async_maybe_transform(
                {
                    "access_level": access_level,
                    "logo": logo,
                    "name": name,
                    "order": order,
                    "section_id": section_id,
                },
                experience_update_params.ExperienceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        app_id: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        product_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ExperienceListResponse, AsyncCursorPage[ExperienceListResponse]]:
        """
        Lists experiences for a company

        Required permissions:

        - `experience:hidden_experience:read`

        Args:
          company_id: The ID of the company to filter experiences by

          after: Returns the elements in the list that come after the specified cursor.

          app_id: The ID of the app to filter experiences by

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          product_id: The ID of the product to filter experiences by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/experiences",
            page=AsyncCursorPage[ExperienceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "app_id": app_id,
                        "before": before,
                        "first": first,
                        "last": last,
                        "product_id": product_id,
                    },
                    experience_list_params.ExperienceListParams,
                ),
            ),
            model=ExperienceListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExperienceDeleteResponse:
        """
        Required permissions:

        - `experience:delete`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/experiences/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceDeleteResponse,
        )

    async def attach(
        self,
        id: str,
        *,
        product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Adds an experience to an product, making it accessible to the product's
        customers.

        Required permissions:

        - `experience:attach`

        Args:
          product_id: The ID of the Access Pass to add the Experience to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/experiences/{id}/attach",
            body=await async_maybe_transform(
                {"product_id": product_id}, experience_attach_params.ExperienceAttachParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )

    async def detach(
        self,
        id: str,
        *,
        product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experience:
        """
        Removes an experience from an product, making it inaccessible to the product's
        customers.

        Required permissions:

        - `experience:detach`

        Args:
          product_id: The ID of the Access Pass to add the Experience to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/experiences/{id}/detach",
            body=await async_maybe_transform(
                {"product_id": product_id}, experience_detach_params.ExperienceDetachParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experience,
        )


class ExperiencesResourceWithRawResponse:
    def __init__(self, experiences: ExperiencesResource) -> None:
        self._experiences = experiences

        self.create = to_raw_response_wrapper(
            experiences.create,
        )
        self.retrieve = to_raw_response_wrapper(
            experiences.retrieve,
        )
        self.update = to_raw_response_wrapper(
            experiences.update,
        )
        self.list = to_raw_response_wrapper(
            experiences.list,
        )
        self.delete = to_raw_response_wrapper(
            experiences.delete,
        )
        self.attach = to_raw_response_wrapper(
            experiences.attach,
        )
        self.detach = to_raw_response_wrapper(
            experiences.detach,
        )


class AsyncExperiencesResourceWithRawResponse:
    def __init__(self, experiences: AsyncExperiencesResource) -> None:
        self._experiences = experiences

        self.create = async_to_raw_response_wrapper(
            experiences.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            experiences.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            experiences.update,
        )
        self.list = async_to_raw_response_wrapper(
            experiences.list,
        )
        self.delete = async_to_raw_response_wrapper(
            experiences.delete,
        )
        self.attach = async_to_raw_response_wrapper(
            experiences.attach,
        )
        self.detach = async_to_raw_response_wrapper(
            experiences.detach,
        )


class ExperiencesResourceWithStreamingResponse:
    def __init__(self, experiences: ExperiencesResource) -> None:
        self._experiences = experiences

        self.create = to_streamed_response_wrapper(
            experiences.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            experiences.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            experiences.update,
        )
        self.list = to_streamed_response_wrapper(
            experiences.list,
        )
        self.delete = to_streamed_response_wrapper(
            experiences.delete,
        )
        self.attach = to_streamed_response_wrapper(
            experiences.attach,
        )
        self.detach = to_streamed_response_wrapper(
            experiences.detach,
        )


class AsyncExperiencesResourceWithStreamingResponse:
    def __init__(self, experiences: AsyncExperiencesResource) -> None:
        self._experiences = experiences

        self.create = async_to_streamed_response_wrapper(
            experiences.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            experiences.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            experiences.update,
        )
        self.list = async_to_streamed_response_wrapper(
            experiences.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            experiences.delete,
        )
        self.attach = async_to_streamed_response_wrapper(
            experiences.attach,
        )
        self.detach = async_to_streamed_response_wrapper(
            experiences.detach,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import (
    InvoiceVoidResponse,
    InvoiceCreateResponse,
)
from whop_sdk._utils import parse_datetime
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import Invoice, InvoiceListItem

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInvoices:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Whop) -> None:
        invoice = client.invoices.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={},
        )
        assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Whop) -> None:
        invoice = client.invoices.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={
                "billing_period": 42,
                "custom_fields": [
                    {
                        "field_type": "text",
                        "name": "name",
                        "id": "id",
                        "order": 42,
                        "placeholder": "placeholder",
                        "required": True,
                    }
                ],
                "description": "description",
                "expiration_days": 42,
                "initial_price": 6.9,
                "internal_notes": "internal_notes",
                "plan_type": "renewal",
                "release_method": "buy_now",
                "renewal_price": 6.9,
                "trial_period_days": 42,
                "visibility": "visible",
            },
            charge_buyer_fee=True,
            customer_name="customer_name",
            email_address="email_address",
            member_id="mber_xxxxxxxxxxxxx",
            payment_token_id="payt_xxxxxxxxxxxxx",
            product={
                "title": "title",
                "product_tax_code_id": "ptc_xxxxxxxxxxxxxx",
            },
            product_id="prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Whop) -> None:
        response = client.invoices.with_raw_response.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = response.parse()
        assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Whop) -> None:
        with client.invoices.with_streaming_response.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = response.parse()
            assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        invoice = client.invoices.retrieve(
            "inv_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Invoice, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.invoices.with_raw_response.retrieve(
            "inv_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = response.parse()
        assert_matches_type(Invoice, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.invoices.with_streaming_response.retrieve(
            "inv_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = response.parse()
            assert_matches_type(Invoice, invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.invoices.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        invoice = client.invoices.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[InvoiceListItem], invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        invoice = client.invoices.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            direction="asc",
            filters={
                "access_pass_ids": ["string"],
                "collection_methods": ["send_invoice"],
                "statuses": ["open"],
            },
            first=42,
            last=42,
            order="id",
        )
        assert_matches_type(SyncCursorPage[InvoiceListItem], invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.invoices.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = response.parse()
        assert_matches_type(SyncCursorPage[InvoiceListItem], invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.invoices.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = response.parse()
            assert_matches_type(SyncCursorPage[InvoiceListItem], invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_void(self, client: Whop) -> None:
        invoice = client.invoices.void(
            "inv_xxxxxxxxxxxxxx",
        )
        assert_matches_type(InvoiceVoidResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_void(self, client: Whop) -> None:
        response = client.invoices.with_raw_response.void(
            "inv_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = response.parse()
        assert_matches_type(InvoiceVoidResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_void(self, client: Whop) -> None:
        with client.invoices.with_streaming_response.void(
            "inv_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = response.parse()
            assert_matches_type(InvoiceVoidResponse, invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_void(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.invoices.with_raw_response.void(
                "",
            )


class TestAsyncInvoices:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncWhop) -> None:
        invoice = await async_client.invoices.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={},
        )
        assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncWhop) -> None:
        invoice = await async_client.invoices.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={
                "billing_period": 42,
                "custom_fields": [
                    {
                        "field_type": "text",
                        "name": "name",
                        "id": "id",
                        "order": 42,
                        "placeholder": "placeholder",
                        "required": True,
                    }
                ],
                "description": "description",
                "expiration_days": 42,
                "initial_price": 6.9,
                "internal_notes": "internal_notes",
                "plan_type": "renewal",
                "release_method": "buy_now",
                "renewal_price": 6.9,
                "trial_period_days": 42,
                "visibility": "visible",
            },
            charge_buyer_fee=True,
            customer_name="customer_name",
            email_address="email_address",
            member_id="mber_xxxxxxxxxxxxx",
            payment_token_id="payt_xxxxxxxxxxxxx",
            product={
                "title": "title",
                "product_tax_code_id": "ptc_xxxxxxxxxxxxxx",
            },
            product_id="prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncWhop) -> None:
        response = await async_client.invoices.with_raw_response.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = await response.parse()
        assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncWhop) -> None:
        async with async_client.invoices.with_streaming_response.create(
            collection_method="send_invoice",
            company_id="biz_xxxxxxxxxxxxxx",
            due_date=parse_datetime("2023-12-01T05:00:00.401Z"),
            plan={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = await response.parse()
            assert_matches_type(InvoiceCreateResponse, invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        invoice = await async_client.invoices.retrieve(
            "inv_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Invoice, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.invoices.with_raw_response.retrieve(
            "inv_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = await response.parse()
        assert_matches_type(Invoice, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.invoices.with_streaming_response.retrieve(
            "inv_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = await response.parse()
            assert_matches_type(Invoice, invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.invoices.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        invoice = await async_client.invoices.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[InvoiceListItem], invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        invoice = await async_client.invoices.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            direction="asc",
            filters={
                "access_pass_ids": ["string"],
                "collection_methods": ["send_invoice"],
                "statuses": ["open"],
            },
            first=42,
            last=42,
            order="id",
        )
        assert_matches_type(AsyncCursorPage[InvoiceListItem], invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.invoices.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = await response.parse()
        assert_matches_type(AsyncCursorPage[InvoiceListItem], invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.invoices.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = await response.parse()
            assert_matches_type(AsyncCursorPage[InvoiceListItem], invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_void(self, async_client: AsyncWhop) -> None:
        invoice = await async_client.invoices.void(
            "inv_xxxxxxxxxxxxxx",
        )
        assert_matches_type(InvoiceVoidResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_void(self, async_client: AsyncWhop) -> None:
        response = await async_client.invoices.with_raw_response.void(
            "inv_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice = await response.parse()
        assert_matches_type(InvoiceVoidResponse, invoice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_void(self, async_client: AsyncWhop) -> None:
        async with async_client.invoices.with_streaming_response.void(
            "inv_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice = await response.parse()
            assert_matches_type(InvoiceVoidResponse, invoice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_void(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.invoices.with_raw_response.void(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .shared.custom_cta import CustomCta
from .shared.visibility import Visibility
from .shared.business_types import BusinessTypes
from .shared.industry_types import IndustryTypes
from .shared.global_affiliate_status import GlobalAffiliateStatus

__all__ = ["ProductUpdateParams", "BannerImage"]


class ProductUpdateParams(TypedDict, total=False):
    banner_image: Optional[BannerImage]
    """A banner image for the product in png, jpeg format"""

    business_type: Optional[BusinessTypes]
    """The different business types a company can be."""

    collect_shipping_address: Optional[bool]
    """Whether or not to collect shipping information at checkout from the customer."""

    custom_cta: Optional[CustomCta]
    """The different types of custom CTAs that can be selected."""

    custom_cta_url: Optional[str]
    """The custom call to action URL for the product."""

    custom_statement_descriptor: Optional[str]
    """The custom statement descriptor for the product i.e.

    WHOP\\**SPORTS, must be between 5 and 22 characters, contain at least one letter,
    and not contain any of the following characters: <, >, \\,, ', "
    """

    description: Optional[str]
    """A written description of the product."""

    global_affiliate_percentage: Optional[float]
    """The percentage of the revenue that goes to the global affiliate program."""

    global_affiliate_status: Optional[GlobalAffiliateStatus]
    """The different statuses of the global affiliate program for an access pass."""

    headline: Optional[str]
    """The headline of the product."""

    industry_type: Optional[IndustryTypes]
    """The different industry types a company can be in."""

    member_affiliate_percentage: Optional[float]
    """The percentage of the revenue that goes to the member affiliate program."""

    member_affiliate_status: Optional[GlobalAffiliateStatus]
    """The different statuses of the global affiliate program for an access pass."""

    product_tax_code_id: Optional[str]
    """The ID of the product tax code to apply to this product."""

    redirect_purchase_url: Optional[str]
    """The URL to redirect the customer to after a purchase."""

    route: Optional[str]
    """The route of the product."""

    title: Optional[str]
    """The title of the product."""

    visibility: Optional[Visibility]
    """Visibility of a resource"""


class BannerImage(TypedDict, total=False):
    id: Optional[str]
    """The ID of an existing attachment object.

    Use this when updating a resource and keeping a subset of the attachments. Don't
    use this unless you know what you're doing.
    """

    direct_upload_id: Optional[str]
    """This ID should be used the first time you upload an attachment.

    It is the ID of the direct upload that was created when uploading the file to S3
    via the mediaDirectUpload mutation.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from datetime import datetime, timezone

import pytest
import standardwebhooks

from whop_sdk import Whop

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWebhooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    def test_method_unwrap(self, client: Whop) -> None:
        key = b"secret"
        hook = standardwebhooks.Webhook(key)

        data = """{"id":"msg_xxxxxxxxxxxxxxxxxxxxxxxx","api_version":"v1","data":{"id":"inv_xxxxxxxxxxxxxx","created_at":"2023-12-01T05:00:00.401Z","current_plan":{"id":"plan_xxxxxxxxxxxxx","currency":"usd","formatted_price":"formatted_price"},"due_date":"2023-12-01T05:00:00.401Z","email_address":"email_address","fetch_invoice_token":"fetch_invoice_token","number":"number","status":"open","user":{"id":"user_xxxxxxxxxxxxx","name":"name","username":"username"}},"timestamp":"2025-01-01T00:00:00.000Z","type":"invoice.created"}"""
        msg_id = "1"
        timestamp = datetime.now(tz=timezone.utc)
        sig = hook.sign(msg_id=msg_id, timestamp=timestamp, data=data)
        headers = {
            "webhook-id": msg_id,
            "webhook-timestamp": str(int(timestamp.timestamp())),
            "webhook-signature": sig,
        }

        try:
            _ = client.webhooks.unwrap(data, headers=headers, key=key)
        except standardwebhooks.WebhookVerificationError as e:
            raise AssertionError("Failed to unwrap valid webhook") from e

        bad_headers = [
            {**headers, "webhook-signature": hook.sign(msg_id=msg_id, timestamp=timestamp, data="xxx")},
            {**headers, "webhook-id": "bad"},
            {**headers, "webhook-timestamp": "0"},
        ]
        for bad_header in bad_headers:
            with pytest.raises(standardwebhooks.WebhookVerificationError):
                _ = client.webhooks.unwrap(data, headers=bad_header, key=key)


class TestAsyncWebhooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    def test_method_unwrap(self, client: Whop) -> None:
        key = b"secret"
        hook = standardwebhooks.Webhook(key)

        data = """{"id":"msg_xxxxxxxxxxxxxxxxxxxxxxxx","api_version":"v1","data":{"id":"inv_xxxxxxxxxxxxxx","created_at":"2023-12-01T05:00:00.401Z","current_plan":{"id":"plan_xxxxxxxxxxxxx","currency":"usd","formatted_price":"formatted_price"},"due_date":"2023-12-01T05:00:00.401Z","email_address":"email_address","fetch_invoice_token":"fetch_invoice_token","number":"number","status":"open","user":{"id":"user_xxxxxxxxxxxxx","name":"name","username":"username"}},"timestamp":"2025-01-01T00:00:00.000Z","type":"invoice.created"}"""
        msg_id = "1"
        timestamp = datetime.now(tz=timezone.utc)
        sig = hook.sign(msg_id=msg_id, timestamp=timestamp, data=data)
        headers = {
            "webhook-id": msg_id,
            "webhook-timestamp": str(int(timestamp.timestamp())),
            "webhook-signature": sig,
        }

        try:
            _ = client.webhooks.unwrap(data, headers=headers, key=key)
        except standardwebhooks.WebhookVerificationError as e:
            raise AssertionError("Failed to unwrap valid webhook") from e

        bad_headers = [
            {**headers, "webhook-signature": hook.sign(msg_id=msg_id, timestamp=timestamp, data="xxx")},
            {**headers, "webhook-id": "bad"},
            {**headers, "webhook-timestamp": "0"},
        ]
        for bad_header in bad_headers:
            with pytest.raises(standardwebhooks.WebhookVerificationError):
                _ = client.webhooks.unwrap(data, headers=bad_header, key=key)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .business_types import BusinessTypes
from .industry_types import IndustryTypes

__all__ = ["Company", "Logo", "OwnerUser", "SocialLink"]


class Logo(BaseModel):
    url: Optional[str] = None
    """This is the URL you use to render optimized attachments on the client.

    This should be used for apps.
    """


class OwnerUser(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class SocialLink(BaseModel):
    id: str
    """The ID"""

    url: str
    """The URL"""

    website: Literal["x", "instagram", "facebook", "tiktok", "youtube", "linkedin", "twitch", "website"]
    """The website"""


class Company(BaseModel):
    id: str
    """The ID (tag) of the company."""

    business_type: Optional[BusinessTypes] = None
    """The different business types a company can be."""

    created_at: datetime
    """When the company was created (signed up)"""

    description: Optional[str] = None
    """The creator pitch for the company."""

    industry_type: Optional[IndustryTypes] = None
    """The different industry types a company can be in."""

    logo: Optional[Logo] = None
    """The company's logo."""

    member_count: int
    """The number of members in the company."""

    owner_user: OwnerUser
    """The user who owns this company"""

    published_reviews_count: int
    """The number of reviews that have been published for the company."""

    route: str
    """The slug/route of the company on the Whop site."""

    social_links: List[SocialLink]
    """The social media accounts of the company"""

    title: str
    """The title of the company."""

    updated_at: datetime
    """The time the company was last updated."""

    verified: bool
    """If the company is Whop Verified"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.plan_type import PlanType
from .shared.visibility import Visibility
from .shared.release_method import ReleaseMethod
from .shared.collection_method import CollectionMethod

__all__ = ["InvoiceCreateParams", "Plan", "PlanCustomField", "Product"]


class InvoiceCreateParams(TypedDict, total=False):
    collection_method: Required[CollectionMethod]
    """The method of collection for this invoice.

    If using charge_automatically, you must provide a payment_token.
    """

    company_id: Required[str]
    """The company ID to create this invoice for."""

    due_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The date the invoice is due, if applicable."""

    plan: Required[Plan]
    """The properties of the plan to create for this invoice."""

    charge_buyer_fee: Optional[bool]
    """Whether or not to charge the customer a buyer fee."""

    customer_name: Optional[str]
    """The name of the customer to create this invoice for.

    This is required if you want to create an invoice for a customer who does not
    have a member of your company yet.
    """

    email_address: Optional[str]
    """The email address to create this invoice for.

    This is required if you want to create an invoice for a user who does not have a
    member of your company yet.
    """

    member_id: Optional[str]
    """The member ID to create this invoice for.

    Include this if you want to create an invoice for an existing member. If you do
    not have a member ID, you must provide an email_address and customer_name.
    """

    payment_token_id: Optional[str]
    """The payment token ID to use for this invoice.

    If using charge_automatically, you must provide a payment_token.
    """

    product: Optional[Product]
    """The properties of the product to create for this invoice.

    Include this if you want to create an invoice for a new product.
    """

    product_id: Optional[str]
    """The product ID to create this invoice for.

    Include this if you want to create an invoice for an existing product.
    """


class PlanCustomField(TypedDict, total=False):
    field_type: Required[Literal["text"]]
    """The type of the custom field."""

    name: Required[str]
    """The name of the custom field."""

    id: Optional[str]
    """The ID of the custom field (if being updated)"""

    order: Optional[int]
    """The order of the field."""

    placeholder: Optional[str]
    """The placeholder value of the field."""

    required: Optional[bool]
    """Whether or not the field is required."""


class Plan(TypedDict, total=False):
    billing_period: Optional[int]
    """The interval at which the plan charges (renewal plans)."""

    custom_fields: Optional[Iterable[PlanCustomField]]
    """An array of custom field objects."""

    description: Optional[str]
    """The description of the plan."""

    expiration_days: Optional[int]
    """The interval at which the plan charges (expiration plans)."""

    initial_price: Optional[float]
    """An additional amount charged upon first purchase.

    Use only if a one time payment OR you want to charge an additional amount on top
    of the renewal price. Provided as a number in dollars. Eg: 10.43 for $10.43
    """

    internal_notes: Optional[str]
    """A personal description or notes section for the business."""

    plan_type: Optional[PlanType]
    """The type of plan that can be attached to an access pass"""

    release_method: Optional[ReleaseMethod]
    """The methods of how a plan can be released."""

    renewal_price: Optional[float]
    """The amount the customer is charged every billing period.

    Use only if a recurring payment. Provided as a number in dollars. Eg: 10.43 for
    $10.43
    """

    trial_period_days: Optional[int]
    """The number of free trial days added before a renewal plan."""

    visibility: Optional[Visibility]
    """Visibility of a resource"""


class Product(TypedDict, total=False):
    title: Required[str]
    """The title of the product."""

    product_tax_code_id: Optional[str]
    """The ID of the product tax code to apply to this product."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import (
    ProductDeleteResponse,
)
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import Product, ProductListItem

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProducts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Whop) -> None:
        product = client.products.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Whop) -> None:
        product = client.products.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
            access_pass_type="regular",
            banner_image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            business_type="education_program",
            collect_shipping_address=True,
            custom_cta="get_access",
            custom_cta_url="custom_cta_url",
            custom_statement_descriptor="custom_statement_descriptor",
            description="description",
            experience_ids=["string"],
            global_affiliate_percentage=6.9,
            global_affiliate_status="enabled",
            headline="headline",
            industry_type="trading",
            member_affiliate_percentage=6.9,
            member_affiliate_status="enabled",
            plan_options={
                "base_currency": "usd",
                "billing_period": 42,
                "custom_fields": [
                    {
                        "field_type": "text",
                        "name": "name",
                        "id": "id",
                        "order": 42,
                        "placeholder": "placeholder",
                        "required": True,
                    }
                ],
                "initial_price": 6.9,
                "plan_type": "renewal",
                "release_method": "buy_now",
                "renewal_price": 6.9,
                "visibility": "visible",
            },
            product_highlights=[
                {
                    "content": "content",
                    "highlight_type": "qualification",
                    "title": "title",
                }
            ],
            product_tax_code_id="ptc_xxxxxxxxxxxxxx",
            redirect_purchase_url="redirect_purchase_url",
            route="route",
            visibility="visible",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Whop) -> None:
        response = client.products.with_raw_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Whop) -> None:
        with client.products.with_streaming_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        product = client.products.retrieve(
            "prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.products.with_raw_response.retrieve(
            "prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.products.with_streaming_response.retrieve(
            "prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Whop) -> None:
        product = client.products.update(
            id="prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Whop) -> None:
        product = client.products.update(
            id="prod_xxxxxxxxxxxxx",
            banner_image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            business_type="education_program",
            collect_shipping_address=True,
            custom_cta="get_access",
            custom_cta_url="custom_cta_url",
            custom_statement_descriptor="custom_statement_descriptor",
            description="description",
            global_affiliate_percentage=6.9,
            global_affiliate_status="enabled",
            headline="headline",
            industry_type="trading",
            member_affiliate_percentage=6.9,
            member_affiliate_status="enabled",
            product_tax_code_id="ptc_xxxxxxxxxxxxxx",
            redirect_purchase_url="redirect_purchase_url",
            route="route",
            title="title",
            visibility="visible",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Whop) -> None:
        response = client.products.with_raw_response.update(
            id="prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Whop) -> None:
        with client.products.with_streaming_response.update(
            id="prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        product = client.products.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[ProductListItem], product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        product = client.products.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            direction="asc",
            first=42,
            last=42,
            order="active_memberships_count",
            product_types=["regular"],
            visibilities=["visible"],
        )
        assert_matches_type(SyncCursorPage[ProductListItem], product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.products.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(SyncCursorPage[ProductListItem], product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.products.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(SyncCursorPage[ProductListItem], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Whop) -> None:
        product = client.products.delete(
            "prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(ProductDeleteResponse, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Whop) -> None:
        response = client.products.with_raw_response.delete(
            "prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(ProductDeleteResponse, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Whop) -> None:
        with client.products.with_streaming_response.delete(
            "prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(ProductDeleteResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.delete(
                "",
            )


class TestAsyncProducts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
            access_pass_type="regular",
            banner_image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            business_type="education_program",
            collect_shipping_address=True,
            custom_cta="get_access",
            custom_cta_url="custom_cta_url",
            custom_statement_descriptor="custom_statement_descriptor",
            description="description",
            experience_ids=["string"],
            global_affiliate_percentage=6.9,
            global_affiliate_status="enabled",
            headline="headline",
            industry_type="trading",
            member_affiliate_percentage=6.9,
            member_affiliate_status="enabled",
            plan_options={
                "base_currency": "usd",
                "billing_period": 42,
                "custom_fields": [
                    {
                        "field_type": "text",
                        "name": "name",
                        "id": "id",
                        "order": 42,
                        "placeholder": "placeholder",
                        "required": True,
                    }
                ],
                "initial_price": 6.9,
                "plan_type": "renewal",
                "release_method": "buy_now",
                "renewal_price": 6.9,
                "visibility": "visible",
            },
            product_highlights=[
                {
                    "content": "content",
                    "highlight_type": "qualification",
                    "title": "title",
                }
            ],
            product_tax_code_id="ptc_xxxxxxxxxxxxxx",
            redirect_purchase_url="redirect_purchase_url",
            route="route",
            visibility="visible",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncWhop) -> None:
        response = await async_client.products.with_raw_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncWhop) -> None:
        async with async_client.products.with_streaming_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            title="title",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.retrieve(
            "prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.products.with_raw_response.retrieve(
            "prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.products.with_streaming_response.retrieve(
            "prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.update(
            id="prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.update(
            id="prod_xxxxxxxxxxxxx",
            banner_image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            business_type="education_program",
            collect_shipping_address=True,
            custom_cta="get_access",
            custom_cta_url="custom_cta_url",
            custom_statement_descriptor="custom_statement_descriptor",
            description="description",
            global_affiliate_percentage=6.9,
            global_affiliate_status="enabled",
            headline="headline",
            industry_type="trading",
            member_affiliate_percentage=6.9,
            member_affiliate_status="enabled",
            product_tax_code_id="ptc_xxxxxxxxxxxxxx",
            redirect_purchase_url="redirect_purchase_url",
            route="route",
            title="title",
            visibility="visible",
        )
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncWhop) -> None:
        response = await async_client.products.with_raw_response.update(
            id="prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncWhop) -> None:
        async with async_client.products.with_streaming_response.update(
            id="prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[ProductListItem], product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            direction="asc",
            first=42,
            last=42,
            order="active_memberships_count",
            product_types=["regular"],
            visibilities=["visible"],
        )
        assert_matches_type(AsyncCursorPage[ProductListItem], product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.products.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(AsyncCursorPage[ProductListItem], product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.products.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(AsyncCursorPage[ProductListItem], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncWhop) -> None:
        product = await async_client.products.delete(
            "prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(ProductDeleteResponse, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncWhop) -> None:
        response = await async_client.products.with_raw_response.delete(
            "prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(ProductDeleteResponse, product, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncWhop) -> None:
        async with async_client.products.with_streaming_response.delete(
            "prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(ProductDeleteResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["PlanType"]

PlanType: TypeAlias = Literal["renewal", "one_time"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .shared.currency import Currency

__all__ = ["TransferListResponse"]


class TransferListResponse(BaseModel):
    id: str
    """The unique identifier of the credit transaction transfer"""

    amount: float
    """The amount of the credit transaction transfer"""

    created_at: datetime
    """The timestamp when the credit transaction transfer was created"""

    currency: Currency
    """The currency of the credit transaction transfer"""

    destination_ledger_account_id: str
    """The ID of the destination ledger account"""

    fee_amount: Optional[float] = None
    """The decimal fee of the credit transaction transfer"""

    notes: Optional[str] = None
    """The notes of the credit transaction transfer"""

    origin_ledger_account_id: str
    """The ID of the origin ledger account"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ReleaseMethod"]

ReleaseMethod: TypeAlias = Literal["buy_now", "waitlist"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .shared.currency import Currency
from .shared.tax_type import TaxType
from .shared.visibility import Visibility

__all__ = ["PlanUpdateParams", "CustomField", "Image"]


class PlanUpdateParams(TypedDict, total=False):
    billing_period: Optional[int]
    """The interval at which the plan charges (renewal plans)."""

    currency: Optional[Currency]
    """The available currencies on the platform"""

    custom_fields: Optional[Iterable[CustomField]]
    """An array of custom field objects."""

    description: Optional[str]
    """The description of the plan."""

    expiration_days: Optional[int]
    """The interval at which the plan charges (expiration plans)."""

    image: Optional[Image]
    """An image for the plan. This will be visible on the product page to customers."""

    initial_price: Optional[float]
    """An additional amount charged upon first purchase."""

    internal_notes: Optional[str]
    """A personal description or notes section for the business."""

    offer_cancel_discount: Optional[bool]
    """Whether or not to offer a discount to cancel a subscription."""

    override_tax_type: Optional[TaxType]
    """
    Whether or not the tax is included in a plan's price (or if it hasn't been set
    up)
    """

    renewal_price: Optional[float]
    """The amount the customer is charged every billing period."""

    title: Optional[str]
    """The title of the plan. This will be visible on the product page to customers."""

    trial_period_days: Optional[int]
    """The number of free trial days added before a renewal plan."""

    visibility: Optional[Visibility]
    """Visibility of a resource"""


class CustomField(TypedDict, total=False):
    field_type: Required[Literal["text"]]
    """The type of the custom field."""

    name: Required[str]
    """The name of the custom field."""

    id: Optional[str]
    """The ID of the custom field (if being updated)"""

    order: Optional[int]
    """The order of the field."""

    placeholder: Optional[str]
    """The placeholder value of the field."""

    required: Optional[bool]
    """Whether or not the field is required."""


class Image(TypedDict, total=False):
    id: Optional[str]
    """The ID of an existing attachment object.

    Use this when updating a resource and keeping a subset of the attachments. Don't
    use this unless you know what you're doing.
    """

    direct_upload_id: Optional[str]
    """This ID should be used the first time you upload an attachment.

    It is the ID of the direct upload that was created when uploading the file to S3
    via the mediaDirectUpload mutation.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import forum_list_params, forum_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.forum import Forum
from ..types.forum_list_response import ForumListResponse
from ..types.shared.who_can_post_types import WhoCanPostTypes
from ..types.shared.who_can_comment_types import WhoCanCommentTypes
from ..types.shared.email_notification_preferences import EmailNotificationPreferences

__all__ = ["ForumsResource", "AsyncForumsResource"]


class ForumsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ForumsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return ForumsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ForumsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return ForumsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Forum:
        """
        Retrieves a forum

        Required permissions:

        - `forum:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/forums/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Forum,
        )

    def update(
        self,
        id: str,
        *,
        email_notification_preference: Optional[EmailNotificationPreferences] | Omit = omit,
        who_can_comment: Optional[WhoCanCommentTypes] | Omit = omit,
        who_can_post: Optional[WhoCanPostTypes] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Forum:
        """
        Updates a forum

        Required permissions:

        - `forum:moderate`

        Args:
          email_notification_preference: Email notification preference option for a forum feed

          who_can_comment: Who can comment on a forum feed

          who_can_post: Who can post on a forum feed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/forums/{id}",
            body=maybe_transform(
                {
                    "email_notification_preference": email_notification_preference,
                    "who_can_comment": who_can_comment,
                    "who_can_post": who_can_post,
                },
                forum_update_params.ForumUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Forum,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        product_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[ForumListResponse]:
        """
        Lists forums inside a company

        Required permissions:

        - `forum:read`

        Args:
          company_id: The ID of the company to list forums for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          product_id: If provided, only forums connected to this product are returned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/forums",
            page=SyncCursorPage[ForumListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "product_id": product_id,
                    },
                    forum_list_params.ForumListParams,
                ),
            ),
            model=ForumListResponse,
        )


class AsyncForumsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncForumsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncForumsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncForumsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncForumsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Forum:
        """
        Retrieves a forum

        Required permissions:

        - `forum:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/forums/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Forum,
        )

    async def update(
        self,
        id: str,
        *,
        email_notification_preference: Optional[EmailNotificationPreferences] | Omit = omit,
        who_can_comment: Optional[WhoCanCommentTypes] | Omit = omit,
        who_can_post: Optional[WhoCanPostTypes] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Forum:
        """
        Updates a forum

        Required permissions:

        - `forum:moderate`

        Args:
          email_notification_preference: Email notification preference option for a forum feed

          who_can_comment: Who can comment on a forum feed

          who_can_post: Who can post on a forum feed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/forums/{id}",
            body=await async_maybe_transform(
                {
                    "email_notification_preference": email_notification_preference,
                    "who_can_comment": who_can_comment,
                    "who_can_post": who_can_post,
                },
                forum_update_params.ForumUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Forum,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        product_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ForumListResponse, AsyncCursorPage[ForumListResponse]]:
        """
        Lists forums inside a company

        Required permissions:

        - `forum:read`

        Args:
          company_id: The ID of the company to list forums for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          product_id: If provided, only forums connected to this product are returned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/forums",
            page=AsyncCursorPage[ForumListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "product_id": product_id,
                    },
                    forum_list_params.ForumListParams,
                ),
            ),
            model=ForumListResponse,
        )


class ForumsResourceWithRawResponse:
    def __init__(self, forums: ForumsResource) -> None:
        self._forums = forums

        self.retrieve = to_raw_response_wrapper(
            forums.retrieve,
        )
        self.update = to_raw_response_wrapper(
            forums.update,
        )
        self.list = to_raw_response_wrapper(
            forums.list,
        )


class AsyncForumsResourceWithRawResponse:
    def __init__(self, forums: AsyncForumsResource) -> None:
        self._forums = forums

        self.retrieve = async_to_raw_response_wrapper(
            forums.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            forums.update,
        )
        self.list = async_to_raw_response_wrapper(
            forums.list,
        )


class ForumsResourceWithStreamingResponse:
    def __init__(self, forums: ForumsResource) -> None:
        self._forums = forums

        self.retrieve = to_streamed_response_wrapper(
            forums.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            forums.update,
        )
        self.list = to_streamed_response_wrapper(
            forums.list,
        )


class AsyncForumsResourceWithStreamingResponse:
    def __init__(self, forums: AsyncForumsResource) -> None:
        self._forums = forums

        self.retrieve = async_to_streamed_response_wrapper(
            forums.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            forums.update,
        )
        self.list = async_to_streamed_response_wrapper(
            forums.list,
        )
